<?xml version="1.0"?>
<xsl:stylesheet version="1.0" 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:msxml="urn:schemas-microsoft-com:xslt"
  xmlns:state="http://www.vbxml.com/state" 
  xmlns:str="http://www.vbxml.com/str"   >
<xsl:import href="scripts.xsl" />
<xsl:output method="xml" indent="yes"/>
<xsl:param name="ModuleName"></xsl:param>
<xsl:param name="HHP_file"></xsl:param>
<xsl:param name="HHC_file"></xsl:param>
<xsl:param name="HHK_file"></xsl:param>
<xsl:param name="BaseDir"></xsl:param>
<xsl:param name="OverWrite"></xsl:param>
<xsl:param name="CompanyLogo">(C) 1999-2001 Knowledge S.A. All rights reserved</xsl:param>
<xsl:param name="CompanyHref">http://www.knowledge.gr</xsl:param>

 <xsl:template match="/">
   <!--  Call main template to create files -->
   <xsl:apply-templates select="TypeLibInfo" mode="FilesType"/>
   <!--  create and save contents file -->
   <xsl:variable name="LibContents">
     <xsl:apply-templates select="TypeLibInfo" mode="ContentFile"/>
   </xsl:variable>
   <xsl:variable name="varSaveLibConts" 
       select="state:SaveLibContentsInHHC_HHK($LibContents,concat($BaseDir,$HHC_file))" />
   <!--  create and save index file -->
   <xsl:variable name="LibIndex">
     <xsl:apply-templates select="TypeLibInfo" mode="IndexFile"/>
   </xsl:variable>
   <xsl:variable name="varSaveLibIndex" 
       select="state:SaveLibContentsInHHC_HHK($LibIndex,concat($BaseDir,$HHK_file))" />
</xsl:template> 

<!--  Main template to create files -->
<xsl:template match="TypeLibInfo" mode="FilesType">
<!-- Enumerate coclasses  -->
 <xsl:for-each select="CoClasses/CoClass">
 <xsl:variable name="CoClassRelDir" select="concat(@Name,'\')" />
   <xsl:call-template name="ProcessCoclass" >
     <xsl:with-param name="FilesType">HTMLfiles</xsl:with-param>
   </xsl:call-template>
  <!-- ==== Enumerate interfaces ===== -->
 <xsl:for-each select="Interfaces/Interface">
   <xsl:variable name="InterfaceRelDir" select="concat($CoClassRelDir,@Name,'\')" />
   <xsl:call-template name="ProcessInterface" >
     <xsl:with-param name="InterfaceRelDir" select="$InterfaceRelDir"></xsl:with-param>
     <xsl:with-param name="FilesType">HTMLfiles</xsl:with-param>
   </xsl:call-template>
  <!-- ==== Enumerate members ===== -->
 <xsl:for-each select="Members/Member[@Name]">
   <xsl:call-template name="ProcessMember" >
     <xsl:with-param name="InterfaceRelDir" select="$InterfaceRelDir"></xsl:with-param>
     <xsl:with-param name="FilesType">HTMLfiles</xsl:with-param>
   </xsl:call-template>
 </xsl:for-each>
 </xsl:for-each>
</xsl:for-each>
<!-- ==== Enumerate enumerations ! ===== -->
<xsl:if test="Enumerations/Enumeration">
<xsl:for-each select="Enumerations">
<xsl:call-template name="ProcessEnumerations" >
<xsl:with-param name="FilesType">HTMLfiles</xsl:with-param>
</xsl:call-template>
<xsl:for-each select="Enumeration">
   <xsl:call-template name="ProcessEnumeration" >
     <xsl:with-param name="FilesType">HTMLfiles</xsl:with-param>
   </xsl:call-template>
</xsl:for-each>
</xsl:for-each>
</xsl:if>
<xsl:variable name="varSaveList" 
    select="state:SaveListInHHP(concat($BaseDir,$HHP_file))" />
</xsl:template>

<!-- +++++++++ Template to create content file (HHC) +++++++ -->
<xsl:template match="TypeLibInfo" mode="ContentFile">
<BODY>
<UL>
<!-- Enumerate coclasses -->
 <xsl:for-each select="CoClasses/CoClass">
 <xsl:variable name="CoClassRelDir" select="concat(@Name,'\')" />
   <xsl:call-template name="ProcessCoclass" >
     <xsl:with-param name="FilesType">ContentFile</xsl:with-param>
   </xsl:call-template>
  <!-- ==== Enumerate interfaces ===== -->
 <UL>
 <xsl:for-each select="Interfaces/Interface">
   <xsl:variable name="InterfaceRelDir" select="concat($CoClassRelDir,@Name,'\')" />
   <xsl:call-template name="ProcessInterface" >
     <xsl:with-param name="InterfaceRelDir" select="$InterfaceRelDir"></xsl:with-param>
     <xsl:with-param name="FilesType">ContentFile</xsl:with-param>
   </xsl:call-template>
  <!-- ==== Enumerate members ===== -->
 <UL>
 <xsl:for-each select="Members/Member[@Name]">
   <xsl:call-template name="ProcessMember" >
     <xsl:with-param name="InterfaceRelDir" select="$InterfaceRelDir"></xsl:with-param>
     <xsl:with-param name="FilesType">ContentFile</xsl:with-param>
   </xsl:call-template>
 </xsl:for-each>
  </UL>
 </xsl:for-each>
</UL>
</xsl:for-each>

<xsl:if test="Enumerations/Enumeration">
<xsl:for-each select="Enumerations">
<xsl:call-template name="ProcessEnumerations" >
<xsl:with-param name="FilesType">ContentFile</xsl:with-param>
</xsl:call-template>
<UL>
<xsl:for-each select="Enumeration">
   <xsl:call-template name="ProcessEnumeration" >
     <xsl:with-param name="FilesType">ContentFile</xsl:with-param>
   </xsl:call-template>
</xsl:for-each>
</UL>
</xsl:for-each>
</xsl:if>

</UL>
</BODY>
</xsl:template>

<!-- +++++++++ Template to create index file (HHK) +++++++ -->
<xsl:template match="TypeLibInfo" mode="IndexFile">
<BODY>
<UL>
<!-- Enumerate coclasses -->
 <xsl:for-each select="CoClasses/CoClass">
 <xsl:variable name="CoClassRelDir" select="concat(@Name,'\')" />
   <xsl:call-template name="ProcessCoclass" >
     <xsl:with-param name="FilesType">ContentFile</xsl:with-param>
   </xsl:call-template>
  <!-- ==== Enumerate interfaces ===== -->
 <xsl:for-each select="Interfaces/Interface">
   <xsl:variable name="InterfaceRelDir" select="concat($CoClassRelDir,@Name,'\')" />
   <xsl:call-template name="ProcessInterface" >
     <xsl:with-param name="InterfaceRelDir" select="$InterfaceRelDir"></xsl:with-param>
     <xsl:with-param name="FilesType">ContentFile</xsl:with-param>
   </xsl:call-template>
  <!-- ==== Enumerate members ===== -->
 <xsl:for-each select="Members/Member[@Name]">
   <xsl:call-template name="ProcessMember" >
     <xsl:with-param name="InterfaceRelDir" select="$InterfaceRelDir"></xsl:with-param>
     <xsl:with-param name="FilesType">ContentFile</xsl:with-param>
   </xsl:call-template>
 </xsl:for-each>
 </xsl:for-each>
</xsl:for-each>
</UL>
</BODY>
</xsl:template>

<!-- ******************** PROCESS ITEMS *********************** -->
<xsl:template name="ProcessCoclass">
<xsl:param name="FilesType"></xsl:param> 
 <xsl:variable name="CoClassRelDir" select="concat(@Name,'\')" />
 <xsl:variable name="CoClassDir" select="concat($BaseDir,$CoClassRelDir)" />
 <xsl:variable name="CoClassFile" select="concat('CoClass_',@Name,'.html')" />
 <xsl:variable name="CoClassRelFile" select="concat($CoClassRelDir,$CoClassFile)" />
  <xsl:choose>
  <xsl:when test="$FilesType='HTMLfiles'">
  <xsl:variable name="CoClassHtml">
    <xsl:apply-templates select="." >
    </xsl:apply-templates>
  </xsl:variable>
   <xsl:call-template name="SaveHTMLFile_AddToList" >
     <xsl:with-param name="Dir" select="$CoClassDir"></xsl:with-param>
     <xsl:with-param name="File" select="$CoClassFile"></xsl:with-param>
     <xsl:with-param name="Html" select="$CoClassHtml"></xsl:with-param>
     <xsl:with-param name="RelFile" select="$CoClassRelFile"></xsl:with-param>
   </xsl:call-template>
   </xsl:when>
   <xsl:when test="$FilesType='ContentFile'">
	<xsl:call-template name="ContentItem">
	 <xsl:with-param name="Name" select="concat('Object ',@Name)"></xsl:with-param> 
	 <xsl:with-param name="Local" select="$CoClassRelFile"></xsl:with-param> 
	</xsl:call-template>
   </xsl:when>
  </xsl:choose>
</xsl:template>

<xsl:template name="ProcessEnumerations">
<xsl:param name="FilesType"></xsl:param> 
 <xsl:variable name="EnumerationsRelDir" select="concat($ModuleName,'_enumerations\')" />
 <xsl:variable name="EnumerationsDir" select="concat($BaseDir,$EnumerationsRelDir)" />
 <xsl:variable name="EnumerationsFile" select="concat($ModuleName,'_enumerations','.html')" />
 <xsl:variable name="EnumerationsRelFile" select="concat($EnumerationsRelDir,$EnumerationsFile)" />
  <xsl:choose>
  <xsl:when test="$FilesType='HTMLfiles'">
  <xsl:variable name="EnumerationsHtml">
    <xsl:apply-templates select="." >
    </xsl:apply-templates>
  </xsl:variable>
   <xsl:call-template name="SaveHTMLFile_AddToList" >
     <xsl:with-param name="Dir" select="$EnumerationsDir"></xsl:with-param>
     <xsl:with-param name="File" select="$EnumerationsFile"></xsl:with-param>
     <xsl:with-param name="Html" select="$EnumerationsHtml"></xsl:with-param>
     <xsl:with-param name="RelFile" select="$EnumerationsRelFile"></xsl:with-param>
   </xsl:call-template>
   </xsl:when>
   <xsl:when test="$FilesType='ContentFile'">
	<xsl:call-template name="ContentItem">
	 <xsl:with-param name="Name" select="'Enumerations'"></xsl:with-param> 
	 <xsl:with-param name="Local" select="$EnumerationsRelFile"></xsl:with-param> 
	</xsl:call-template>
   </xsl:when>
  </xsl:choose>
</xsl:template>

<xsl:template name="ProcessEnumeration">
<xsl:param name="FilesType"></xsl:param> 
 <xsl:variable name="EnumerationRelDir" select="concat($ModuleName,'_enumerations\')" />
 <xsl:variable name="EnumerationDir" select="concat($BaseDir,$EnumerationRelDir)" />
 <xsl:variable name="EnumerationFile" select="concat('Enumeration_',@Name,'.html')" />
 <xsl:variable name="EnumerationRelFile" select="concat($EnumerationRelDir,$EnumerationFile)" />
  <xsl:choose>
  <xsl:when test="$FilesType='HTMLfiles'">
  <xsl:variable name="EnumerationHtml">
    <xsl:apply-templates select="." >
    </xsl:apply-templates>
  </xsl:variable>
   <xsl:call-template name="SaveHTMLFile_AddToList" >
     <xsl:with-param name="Dir" select="$EnumerationDir"></xsl:with-param>
     <xsl:with-param name="File" select="$EnumerationFile"></xsl:with-param>
     <xsl:with-param name="Html" select="$EnumerationHtml"></xsl:with-param>
     <xsl:with-param name="RelFile" select="$EnumerationRelFile"></xsl:with-param>
   </xsl:call-template>
   </xsl:when>
   <xsl:when test="$FilesType='ContentFile'">
	<xsl:call-template name="ContentItem">
	 <xsl:with-param name="Name" select="concat('Enumeration ',@Name)"></xsl:with-param> 
	 <xsl:with-param name="Local" select="$EnumerationRelFile"></xsl:with-param> 
	</xsl:call-template>
   </xsl:when>
  </xsl:choose>
</xsl:template>

<xsl:template name="ProcessInterface">
<xsl:param name="FilesType"></xsl:param> 
<xsl:param name="InterfaceRelDir"></xsl:param>
 <xsl:variable name="InterfaceDir" select="concat($BaseDir,$InterfaceRelDir)" />
 <!-- Interface description -->
 <xsl:variable name="InterfaceFile" select="concat('Interface_',@Name,'.html')" />
 <xsl:variable name="InterfaceRelFile" select="concat($InterfaceRelDir,$InterfaceFile)" />
 <xsl:variable name="InterfaceMembersFile" select="concat('Interface_',@Name,'_Members.html')" />
 <xsl:variable name="InterfaceMembersRelFile" select="concat($InterfaceRelDir,$InterfaceMembersFile)" />
 <xsl:variable name="InterfaceMembersByCatFile" select="concat('Interface_',@Name,'_Members_ByCategory.html')" />
 <xsl:variable name="InterfaceMembersByCatRelFile" select="concat($InterfaceRelDir,$InterfaceMembersByCatFile)" />
 <xsl:variable name="InterfaceMembersByUseFile" select="concat('Interface_',@Name,'_Members_ByUse.html')" />
 <xsl:variable name="InterfaceMembersByUseRelFile" select="concat($InterfaceRelDir,$InterfaceMembersByUseFile)" />
<xsl:choose>
<xsl:when test="$FilesType='HTMLfiles'">
<xsl:variable name="InterfaceHtml">
   <xsl:call-template name="Interface">
    <xsl:with-param name="Type">Description</xsl:with-param>
    <xsl:with-param name="MembersFile" select="$InterfaceMembersFile"></xsl:with-param>
    <xsl:with-param name="MembersByCatFile" select="$InterfaceMembersByCatFile"></xsl:with-param>
    <xsl:with-param name="MembersByRelFile" select="$InterfaceMembersByUseFile"></xsl:with-param>
   </xsl:call-template> 
 </xsl:variable>
   <xsl:call-template name="SaveHTMLFile_AddToList" >
     <xsl:with-param name="Dir" select="$InterfaceDir"></xsl:with-param>
     <xsl:with-param name="File" select="$InterfaceFile"></xsl:with-param>
     <xsl:with-param name="Html" select="$InterfaceHtml"></xsl:with-param>
     <xsl:with-param name="RelFile" select="$InterfaceRelFile"></xsl:with-param>
   </xsl:call-template>
 <!-- Interface members -->
 <xsl:variable name="InterfaceMembersHtml">
   <xsl:call-template name="Interface">
    <xsl:with-param name="Type">Members</xsl:with-param>
    <xsl:with-param name="DescFile" select="$InterfaceFile"></xsl:with-param>
   </xsl:call-template> 
 </xsl:variable>
   <xsl:call-template name="SaveHTMLFile_AddToList" >
     <xsl:with-param name="Dir" select="$InterfaceDir"></xsl:with-param>
     <xsl:with-param name="File" select="$InterfaceMembersFile"></xsl:with-param>
     <xsl:with-param name="Html" select="$InterfaceMembersHtml"></xsl:with-param>
     <xsl:with-param name="RelFile" select="$InterfaceMembersRelFile"></xsl:with-param>
   </xsl:call-template>
 <!-- Interface members by category -->
 <xsl:variable name="InterfaceMembersByCatHtml">
   <xsl:call-template name="Interface">
    <xsl:with-param name="Type">Members</xsl:with-param>
    <xsl:with-param name="DescFile" select="$InterfaceFile"></xsl:with-param>
    <xsl:with-param name="SortParam">CategoryID</xsl:with-param>
   </xsl:call-template> 
 </xsl:variable>
   <xsl:call-template name="SaveHTMLFile_AddToList" >
     <xsl:with-param name="Dir" select="$InterfaceDir"></xsl:with-param>
     <xsl:with-param name="File" select="$InterfaceMembersByCatFile"></xsl:with-param>
     <xsl:with-param name="Html" select="$InterfaceMembersByCatHtml"></xsl:with-param>
     <xsl:with-param name="RelFile" select="$InterfaceMembersByCatRelFile"></xsl:with-param>
   </xsl:call-template>
 <!-- Interface members by use -->
 <xsl:variable name="InterfaceMembersByUseHtml">
   <xsl:call-template name="Interface">
    <xsl:with-param name="Type">Members</xsl:with-param>
    <xsl:with-param name="DescFile" select="$InterfaceFile"></xsl:with-param>
    <xsl:with-param name="SortParam">FrequencyOfUseID</xsl:with-param>
   </xsl:call-template> 
 </xsl:variable>
   <xsl:call-template name="SaveHTMLFile_AddToList" >
     <xsl:with-param name="Dir" select="$InterfaceDir"></xsl:with-param>
     <xsl:with-param name="File" select="$InterfaceMembersByUseFile"></xsl:with-param>
     <xsl:with-param name="Html" select="$InterfaceMembersByUseHtml"></xsl:with-param>
     <xsl:with-param name="RelFile" select="$InterfaceMembersByUseRelFile"></xsl:with-param>
   </xsl:call-template>
   </xsl:when>
   <xsl:when test="$FilesType='ContentFile'">
	<xsl:call-template name="ContentItem">
	 <xsl:with-param name="Name" select="concat('Interface ',@Name)"></xsl:with-param> 
	 <xsl:with-param name="Local" select="$InterfaceRelFile"></xsl:with-param> 
	</xsl:call-template>
	<xsl:call-template name="ContentItem">
	 <xsl:with-param name="Name" select="concat(@Name, ' members')"></xsl:with-param> 
	 <xsl:with-param name="Local" select="$InterfaceMembersRelFile"></xsl:with-param> 
	</xsl:call-template>
   </xsl:when>
  </xsl:choose>
</xsl:template>

<xsl:template name="ProcessMember">
<xsl:param name="FilesType"></xsl:param> 
<xsl:param name="InterfaceRelDir"></xsl:param> 
 <xsl:variable name="MemberFile" select="concat(@Name,'.html')" />
 <xsl:variable name="MemberRelFile" select="concat($InterfaceRelDir,$MemberFile)" />
 <xsl:variable name="InterfaceMembersFile" select="concat('Interface_',../../@Name,'_Members.html')" />
 <xsl:variable name="InterfaceDir" select="concat($BaseDir,$InterfaceRelDir)" />
 <xsl:choose>
 <xsl:when test="$FilesType='HTMLfiles'">
  <xsl:variable name="MemberHtml">
    <xsl:apply-templates select="." >
    <xsl:with-param name="MembersFile" select="$InterfaceMembersFile"></xsl:with-param>
    </xsl:apply-templates>
  </xsl:variable>
   <xsl:call-template name="SaveHTMLFile_AddToList" >
     <xsl:with-param name="Dir" select="$InterfaceDir"></xsl:with-param>
     <xsl:with-param name="File" select="$MemberFile"></xsl:with-param>
     <xsl:with-param name="Html" select="$MemberHtml"></xsl:with-param>
     <xsl:with-param name="RelFile" select="$MemberRelFile"></xsl:with-param>
   </xsl:call-template>
   </xsl:when>
   <xsl:when test="$FilesType='ContentFile'">
	<xsl:call-template name="ContentItem">
	 <xsl:with-param name="Name" select="concat(@Name,' (',@MemberTypeShort,') ')"></xsl:with-param> 
	 <xsl:with-param name="Local" select="$MemberRelFile"></xsl:with-param> 
	</xsl:call-template>
   </xsl:when>
  </xsl:choose>
</xsl:template>

<!-- ~~~~~~~~ TEMPLATE FOR COCLASSES ~~~~~~~~~ -->
<!-- TODO: Change the format of coclass overview file -->
<xsl:template match="CoClass">
<HTML>
<HEAD>
<meta http-equiv="Content-Type" content="text/html; charset=iso-8859-7" />
<LINK REL="stylesheet" TYPE="text/css" HREF="../ie4.css" />
<TITLE>Coclass <xsl:value-of select="@Name"/></TITLE>
</HEAD>
<BODY>
<p><H2>Coclass: <xsl:value-of select="@Name"/></H2></p>
<p><xsl:value-of select="CoClassDescription"/></p>
</BODY>
</HTML>
</xsl:template>

<!-- TODO: Change the format of Enumerations overview file -->
<xsl:template match="Enumerations">
<HTML>
<HEAD>
<meta http-equiv="Content-Type" content="text/html; charset=iso-8859-7" />
<LINK REL="stylesheet" TYPE="text/css" HREF="../ie4.css" />
<TITLE>Enumerations</TITLE>
</HEAD>
<BODY>
<h2>Enumerations</h2>
<xsl:for-each select="Enumeration">
  <xsl:variable name="EnumLink" select="concat('Enumeration_',@Name,'.html')" />
  <p><A HREF="{$EnumLink}">Enumeration <xsl:value-of select="@Name"/></A></p>
</xsl:for-each> 
</BODY>
</HTML>
</xsl:template>

<!-- TODO: Change the format of Enumeration file -->
<xsl:template match="Enumeration">
<HTML>
<HEAD>
<meta http-equiv="Content-Type" content="text/html; charset=iso-8859-7" />
<LINK REL="stylesheet" TYPE="text/css" HREF="../ie4.css" />
<TITLE>Enumeration <xsl:value-of select="@Name"/></TITLE>
</HEAD>
<BODY>
<h2>Enumeration <xsl:value-of select="@Name"/></h2>
<xsl:value-of select="EnumerationDescription" /><br/>
<TABLE class="BlackBorder" 
  BORDER="1" borderColor="black" cellSpacing="3" cellPadding="3" >
  <TR>
    <TH><EM>Member name</EM></TH>
    <TH><EM>Member value</EM></TH>
  </TR>
<xsl:for-each select="EnumMembers/EnumMember">
  <TR>
    <TD><xsl:value-of select="@Name"/></TD>
    <TD><xsl:value-of select="@Value"/></TD>
  </TR>
</xsl:for-each> 
</TABLE>
<p/>
<INPUT id="button1" size="10" type="button" value="Back to list of enumerations" 
    onclick="javascript:location.href='Enumerations.html'"/>
</BODY>
</HTML>
</xsl:template>

<!-- ~~~~~~~~ TEMPLATE FOR INTERFACES ~~~~~~~~~ -->
<!-- TODO: Change the format of interface overview file -->
<xsl:template name="Interface">
<xsl:param name="Type"></xsl:param> 
<xsl:param name="SortParam"></xsl:param> 
<xsl:param name="DescFile"></xsl:param> 
<xsl:param name="MembersFile"></xsl:param> 
<xsl:param name="MembersByCatFile"></xsl:param> 
<xsl:param name="MembersByRelFile"></xsl:param> 
<HTML>
<HEAD>
<meta http-equiv="Content-Type" content="text/html; charset=iso-8859-7" />
<LINK REL="stylesheet" TYPE="text/css" HREF="../../ie4.css" />
<TITLE>Interface <xsl:value-of select="@Name"/></TITLE>
</HEAD>
<BODY>
<xsl:choose>
 <!-- Sub template for main interface page -->
<xsl:when test="$Type='Description'"> 
<p><H2>Interface: <xsl:value-of select="@Name"/></H2></p>
<p><xsl:value-of select="InterfaceDescription"/></p>
<p><A HREF="{$MembersFile}">Browse members alphabetically</A></p>
<p><A HREF="{$MembersByCatFile}">Browse members by category</A></p>
<p><A HREF="{$MembersByRelFile}">Browse members by use</A></p>
 </xsl:when>  
 <!-- Sub template for list of methods -->
<xsl:when test="$Type='Members'"> 
<p><H2><xsl:value-of select="@Name"/> members
<xsl:if test="$SortParam='CategoryID'"> by category</xsl:if>
<xsl:if test="$SortParam='FrequencyOfUseID'"> by frequency of use</xsl:if>
</H2></p>
   <xsl:call-template name="MemberTable" >
     <xsl:with-param name="Rows" select="Members/Member[@MemberTypeShort]"></xsl:with-param>
     <xsl:with-param name="Title">Properties</xsl:with-param>
     <xsl:with-param name="SortParam" select="$SortParam"></xsl:with-param>
   </xsl:call-template>
<p><A HREF="{$DescFile}"><xsl:value-of select="@Name"/> overview</A></p>
 </xsl:when>  
</xsl:choose>
</BODY>
</HTML>
</xsl:template>

<xsl:template name="MemberTable">
<xsl:param name="Rows"></xsl:param>
<xsl:param name="Title"></xsl:param>
<xsl:param name="SortParam"></xsl:param>
<xsl:if test="count($Rows)>0">
<H2><i><xsl:value-of select="$Title" /></i></H2>
<TABLE class="BlackBorder" 
  BORDER="1" borderColor="black" cellSpacing="3" cellPadding="3" >
    <TH><EM>Name</EM></TH>
    <TH><EM>Type</EM></TH>
    <TH><EM>HelpString</EM></TH>
    <TH><EM>Category</EM></TH>
    <TH><EM>Frequency of use</EM></TH>
 <xsl:choose>
 <xsl:when test="$SortParam='CategoryID'">
 <xsl:for-each select="$Rows/self::Member">
 <xsl:sort select="@CategoryID"/>
 <xsl:call-template name="MemberRow" />
 </xsl:for-each>
 </xsl:when>
<xsl:when test="$SortParam='FrequencyOfUseID'">
 <xsl:for-each select="$Rows/self::Member">
 <xsl:sort select="@FrequencyOfUseID"/>
 <xsl:call-template name="MemberRow" />
 </xsl:for-each>
 </xsl:when>
<xsl:otherwise>
 <xsl:for-each select="$Rows/self::Member">
 <xsl:call-template name="MemberRow" />
 </xsl:for-each>
 </xsl:otherwise>
 </xsl:choose>
</TABLE>
</xsl:if>
</xsl:template>

<xsl:template name="MemberRow">
 <TR>
  <TD><A HREF="{@Name}.html"><xsl:value-of select="@Name"/></A></TD>
  <TD><xsl:value-of select="@MemberType"/></TD>
  <TD><xsl:value-of select="@HelpString"/></TD>
  <TD><xsl:value-of select="@CategoryID"/>
   <xsl:if test="@CategoryID=''">General</xsl:if></TD>
  <TD>
 <xsl:choose>
   <xsl:when test="@FrequencyOfUseID='1'">High</xsl:when>
   <xsl:when test="@FrequencyOfUseID='2'">Normal</xsl:when>
   <xsl:when test="@FrequencyOfUseID='3'">Low</xsl:when>
   <xsl:when test="@FrequencyOfUseID='4'">Rare</xsl:when>
   <xsl:when test="@FrequencyOfUseID='5'">Obsolete</xsl:when>
   <xsl:otherwise>Normal</xsl:otherwise>
 </xsl:choose>
   </TD>
 </TR>
</xsl:template>

<!-- ~~~~~~~~ TEMPLATE FOR METHODS ~~~~~~~~~ -->
<!-- TODO: Change the format of method description file -->
<xsl:template match="Member">
<xsl:param name="MembersFile"></xsl:param> 
<HTML>
<HEAD>
<meta http-equiv="Content-Type" content="text/html; charset=iso-8859-7" />
<LINK REL="stylesheet" TYPE="text/css" HREF="../../ie4.css" />
<TITLE><xsl:value-of select="../../@Name"/>::<xsl:value-of select="@Name"/></TITLE>
</HEAD>
<BODY>

<FONT style="BACKGROUND-COLOR: lightgoldenrodyellow" size="4"> 
<p><H2><xsl:value-of select="../../@Name"/>::<xsl:value-of select="@Name"/></H2></p>
</FONT>
<H4><xsl:value-of select="@HelpString"/></H4><br/>
<CODE><xsl:value-of select="@Declaration"/></CODE><p/>
   <xsl:choose>
   <xsl:when test="Parameters/Parameter">
<H3>Parameters</H3>
<TABLE class="BlackBorder" 
  BORDER="1" borderColor="black" cellSpacing="3" cellPadding="3" >
  <TR>
    <TH><EM>No</EM></TH>
    <TH><EM>Name</EM></TH>
    <TH><EM>Type</EM></TH>
    <TH><EM>Optional</EM></TH>
    <TH><EM>Default</EM></TH>
    <TH><EM>Description</EM></TH></TR>
 <xsl:for-each select="Parameters/Parameter">
  <TR>
    <TD><xsl:value-of select="position()"/></TD>
    <TD><xsl:value-of select="@Name"/></TD>
    <TD><xsl:value-of select="@Type"/></TD>
    <TD><xsl:value-of select="@Optional"/><xsl:if test="@Optional=''">-</xsl:if></TD>
    <TD><xsl:value-of select="@DefValue"/><xsl:if test="@DefValue=''">-</xsl:if></TD>
    <TD><xsl:value-of select="ParameterDescription"/></TD>
    </TR>
   </xsl:for-each>
 </TABLE>
 </xsl:when>
 <xsl:otherwise>
   No parameters
 </xsl:otherwise>
 </xsl:choose>                 
<xsl:if test="not(@ReturnType='') and @MemberTypeShort='Function'"><p><H3>Return value (<xsl:value-of select="@ReturnType"/>)</H3></p>
<xsl:apply-templates select="ReturnValueDescription" /></xsl:if>
<xsl:if test="not(str:trimtext(string(MemberDescription))='')"><p><H3><xsl:value-of select="@MemberTypeShort"/> description</H3></p>
<xsl:apply-templates select="MemberDescription" /></xsl:if>
<xsl:if test="not(str:trimtext(string(ExampleCode))='')"><p><H3>Example code</H3></p></xsl:if>
<P><CODE><FONT style="BACKGROUND-COLOR: white"><pre><xsl:value-of select="str:trimtext(string(ExampleCode))"/></pre></FONT> </CODE></P>
<xsl:if test="SeeAlso/Related">
<xsl:if test="not(str:trimtext(string(SeeAlso))='')">
<P><xsl:call-template name="RelatedMembers" >
     <xsl:with-param name="RelatedList" select="SeeAlso/Related"></xsl:with-param>
   </xsl:call-template></P>
</xsl:if></xsl:if>
<INPUT id="button1" size="10" type="button" value="Back to {../../@Name} members" 
    onclick="javascript:location.href='{$MembersFile}'"/>
<P align="left"><i><A href="{$CompanyHref}"><FONT size="-2"> 
<xsl:value-of select="$CompanyLogo"/></FONT></A></i></P>
</BODY>
</HTML>
</xsl:template>

<xsl:template name="RelatedMembers">
<xsl:param name="RelatedList"></xsl:param> 
<OBJECT id="hhctrl1" type="application/x-oleobject"
        classid="clsid:adb880a6-d8ff-11cf-9377-00aa003b7a11"
        codebase="hhctrl.ocx#Version=4,74,8702,0"
        width="12"
        height="12"
>
    <PARAM name="Command" value="Related Topics"></PARAM>
    <PARAM name="Button" value="Related members"></PARAM>
 <xsl:for-each select="$RelatedList/self::Related">
    <PARAM>
      <xsl:attribute name="name">Item<xsl:value-of select="position()"/></xsl:attribute>
      <xsl:attribute name="value">Member <xsl:value-of select="."/>
        <![CDATA[;]]><xsl:value-of select="."/>.html</xsl:attribute>
    </PARAM>
</xsl:for-each>
</OBJECT>
</xsl:template>

<xsl:template name="ContentItem">
<xsl:param name="Name"></xsl:param> 
<xsl:param name="Local"></xsl:param> 
<LI><OBJECT type="text/sitemap">
  <param name="Name" value="{$Name}"/>
  <param name="Local" value="{$Local}"/></OBJECT></LI>
</xsl:template>

<xsl:template name="SaveHTMLFile_AddToList">
<xsl:param name="Dir">c:\</xsl:param> 
<xsl:param name="File">test.txt</xsl:param> 
<xsl:param name="Html">TEST</xsl:param> 
<xsl:param name="RelFile">REL</xsl:param> 
 <xsl:variable name="persist"
  select="state:saveDoc2File($Dir,$File,$Html,$OverWrite)" />
 <xsl:variable name="AddFile"
    select="state:AddFileToList($RelFile)" /> 
</xsl:template>

<!-- br tag is used as Carriage Return in the method description. Leave it as is -->
<xsl:template match="br|BR">
 <xsl:copy-of select="."/>
</xsl:template>
    
<!-- A tag is used for links Leave it as is -->
<xsl:template match="A|a">
 <xsl:copy-of select="."/>
</xsl:template>

<!-- Table tags Leave  as is -->
<xsl:template match="TABLE|TR|TH|TD|UL|LI|table|tr|td|th|ul|li">
 <xsl:copy-of select="."/>
</xsl:template>

<!-- Format tags Leave  as is -->
<xsl:template match="B|I|U|b|i|u">
 <xsl:copy-of select="."/>
</xsl:template>

</xsl:stylesheet>
